#!/usr/bin/env node
'use strict';

var path = require('path');
var chalk = require('chalk');
var fs = require('fs/promises');
var minimist = require('minimist');
var UpdateNotifier = require('update-notifier');
var cliSpinner = require('cli-spinner');
var Table = require('cli-table');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var chalk__default = /*#__PURE__*/_interopDefaultLegacy(chalk);
var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);
var minimist__default = /*#__PURE__*/_interopDefaultLegacy(minimist);
var UpdateNotifier__default = /*#__PURE__*/_interopDefaultLegacy(UpdateNotifier);
var Table__default = /*#__PURE__*/_interopDefaultLegacy(Table);

const theGit$1 = require('git-state');
const isGit = (path) => new Promise((resolve) => {
    theGit$1.isGit(path, function (result) {
        resolve(result);
    });
});

var name = "list-repos";
var version = "1.0.0-beta.10";
var description = "List all the repos, their HEADs and clean status in the Current Directory or the Supplied Directory";
var main = "dist/index.cjs";
var type = "commonjs";
var engines = {
	node: ">=14.0.0"
};
var scripts = {
	test: "jest",
	"test:watch": "jest --watch",
	"test:coverage": "jest --coverage",
	build: "rollup --config",
	typecheck: "tsc --noEmit --checkJs false",
	prepare: "husky install",
	"pre-commit": "yarn test && lint-staged && yarn run typecheck && yarn build"
};
var types = "./index.d.ts";
var repository = {
	type: "git",
	url: "git+https://github.com/pankajpatel/list-repos.git"
};
var keywords = [
	"git",
	"cli"
];
var bin = {
	"list-repos": "dist/index.cjs"
};
var author = "Pankaj Patel <patel.pankaj@live.in>";
var license = "MIT";
var bugs = {
	url: "https://github.com/pankajpatel/list-repos/issues"
};
var homepage = "https://github.com/pankajpatel/list-repos#readme";
var dependencies = {
	"@rollup/plugin-json": "^4.1.0",
	"@rollup/plugin-typescript": "^8.3.2",
	chalk: "~4.1.2",
	"cli-spinner": "^0.2.5",
	"cli-table": "^0.3.4",
	"git-state": "^4.1.0",
	minimist: "^1.2.0",
	"type-fest": "^2.12.2",
	typescript: "^4.6.3",
	"update-notifier": "^5.1.0"
};
var devDependencies = {
	"@rollup/plugin-node-resolve": "^13.2.1",
	"@types/cli-spinner": "^0.2.1",
	"@types/cli-table": "^0.3.0",
	"@types/jest": "^27.4.1",
	"@types/minimist": "^1.2.2",
	"@types/update-notifier": "^5.1.0",
	"@typescript-eslint/eslint-plugin": "^5.20.0",
	"@typescript-eslint/parser": "^5.20.0",
	eslint: "^8.14.0",
	"eslint-config-prettier": "^8.3.0",
	"eslint-plugin-prettier": "^4.0.0",
	husky: "^7.0.0",
	jest: "^27.4.5",
	"lint-staged": "^12.3.8",
	prettier: "2.6.2",
	rollup: "^2.70.2",
	"rollup-plugin-build-statistics": "^0.0.19",
	"rollup-plugin-delete": "^2.0.0",
	"rollup-plugin-preserve-shebang": "^1.0.1",
	"rollup-plugin-shebang": "^0.1.5",
	"ts-jest": "^27.1.4"
};
var pkg = {
	name: name,
	version: version,
	description: description,
	main: main,
	type: type,
	engines: engines,
	scripts: scripts,
	types: types,
	repository: repository,
	keywords: keywords,
	bin: bin,
	author: author,
	license: license,
	bugs: bugs,
	homepage: homepage,
	dependencies: dependencies,
	devDependencies: devDependencies,
	"lint-staged": {
	"*.js": [
		"eslint",
		"prettier --ignore-unknown --write"
	]
}
};

const optionsText = `
The path defaults to the current directory if not specified.

Options:
  --help, -h            show this help
  --version, -v         show version
  --compact, -c         output compact table
  --compact=s, -c=s     output compact table with short headers
                        with headers described in bottom of table
  --compact=so, -c=so   output compact table with short headers
                        and no description of headers
  --gitonly, -g         output only git repositories
  --attention, -a       output only dirs which requires attention
                        also includes non git dirs, use -g to omit them
  --simple, -s          make the output more simple for easy grepping

`;
const NOOP = () => { };
const COMPACTNESS_LEVELS = {
    NONE: 'NONE',
    LOW: 'LOW',
    MEDIUM: 'MEDIUM',
    HIGH: 'HIGH',
};
const HEADERS = {
    displayPath: {
        long: 'Directory',
        short: 'DIR',
    },
    branch: {
        long: 'Branch',
        short: 'B',
    },
    ahead: {
        long: 'Ahead',
        short: 'A',
    },
    dirty: {
        long: 'Dirty',
        short: 'D',
    },
    untracked: {
        long: 'Untracked',
        short: 'U',
    },
    stashes: {
        long: 'Stashes',
        short: 'S',
    },
};
const SORT_DIRECTIONS = {
    ASC: 'ASC',
    DESC: 'DESC',
};
const SORT_FUNCTIONS = {
    ASC: (a, b) => Number(a[0]) - Number(b[0]),
    DESC: (a, b) => Number(b[0]) - Number(a[0]),
};
var C = {
    defaultHeaderColor: 'cyan',
    updateInterval: 2,
    daysMultiplier: 1000 * 60 * 60 * 24,
    columnsOrder: [
        'displayPath',
        'branch',
        'ahead',
        'dirty',
        'untracked',
        'stashes',
    ],
    headers: HEADERS,
    simple: [
        'displayPath',
        'branch',
        'ahead',
        'dirty',
        'untracked',
        'stashes',
    ],
    emptyGitStatus: {
        branch: '-',
        issues: '-',
        git: false,
        untracked: '-',
        ahead: '-',
        stashes: '-',
        dirty: '-',
    },
    optionsText: optionsText,
    COMPACTNESS_LEVELS,
    NOOP,
};

function getHelp() {
    const lines = [
        `${pkg.name} ${pkg.version}`,
        pkg.description,
        'Usage:',
        `${pkg.name} [path] [options]`,
        optionsText,
    ];
    return lines.join('\n');
}

const theGit = require('git-state');
const gitCheck = (path) => new Promise((resolve, reject) => {
    theGit.check(path, function (error, result) {
        if (error) {
            reject(error);
        }
        resolve(result);
    });
});

const prettyPath = (pathString) => {
    const p = pathString.split(path__default["default"].sep);
    return p[p.length - 1];
};

const COLORS = {
    RED: 'red',
    GREEN: 'green',
    YELLOW: 'yellow',
    GREY: 'grey',
};
const colorForStatus = (state) => {
    if (state.dirty)
        return COLORS.RED;
    if (state.ahead)
        return COLORS.GREEN;
    if (state.untracked)
        return COLORS.YELLOW;
    return COLORS.GREY;
};

const startSpinner = () => {
    const spinner = new cliSpinner.Spinner('%s');
    spinner.setSpinnerString(18);
    spinner.start();
    return spinner;
};
const stopSpinner = (spinner) => {
    spinner.stop();
};

const buildRepoState = (path, repoState) => (Object.assign({ path, displayPath: prettyPath(path), git: Boolean(repoState) }, (repoState || C.emptyGitStatus)));
const processNonGitDir = (stat) => Promise.resolve(buildRepoState(stat.path, null));
const processGitDir = (stat) => gitCheck(stat.path).then((gitStatus) => buildRepoState(stat.path, gitStatus));
const processDirectory = (stat) => {
    var _a;
    if (!stat || !stat.path || !stat.stat) {
        return Promise.resolve(buildRepoState((_a = stat === null || stat === void 0 ? void 0 : stat.path) !== null && _a !== void 0 ? _a : '', null));
    }
    if (!stat.stat.isDirectory()) {
        return Promise.resolve(buildRepoState(stat.path, null));
    }
    return Promise.resolve()
        .catch(() => processNonGitDir(stat))
        .then(() => isGit(stat.path))
        .then((isDirGit) => !isDirGit ? processNonGitDir(stat) : processGitDir(stat));
};
const processDirectories = (stats) => Promise.all(stats.map((status) => processDirectory(status)));

const getTableHeader = (type, color = C.defaultHeaderColor) => C.columnsOrder.map((key) => chalk__default["default"][color](C.headers[key][type]));
const initTable = (compactness) => {
    const tableOpts = {
        head: getTableHeader('long'),
    };
    if (compactness !== COMPACTNESS_LEVELS.NONE) {
        tableOpts.head =
            compactness !== COMPACTNESS_LEVELS.LOW
                ? getTableHeader('short')
                : tableOpts.head;
        tableOpts.chars = {
            mid: '',
            'left-mid': '',
            'mid-mid': '',
            'right-mid': '',
        };
    }
    return new Table__default["default"](tableOpts);
};

const getCompactness = (cliArgs) => {
    const compact = cliArgs.compact || cliArgs.c;
    switch (compact) {
        case 's':
            return C.COMPACTNESS_LEVELS.MEDIUM;
        case 'so':
            return C.COMPACTNESS_LEVELS.HIGH;
        case true:
            return C.COMPACTNESS_LEVELS.LOW;
        default:
            return C.COMPACTNESS_LEVELS.NONE;
    }
};

function checkAndGetEmptyString(status, key, methodName) {
    return chalk__default["default"][methodName](status[key] || '-');
}
const pushToTable = (table, options) => (status) => {
    if (options.showGitOnly && !status.git) {
        return;
    }
    const methodName = colorForStatus(status);
    if (!(options.needsAttention && methodName === 'grey')) {
        table.push(C.columnsOrder.map((key) => key === 'displayPath'
            ? status.displayPath
            : checkAndGetEmptyString(status, key, methodName)));
    }
};

const argv = minimist__default["default"](process.argv.slice(2));
const printLine = console.log;
const showGitOnly = argv.gitonly || argv.g;
const shouldShowHelp = argv.help || argv.h;
const needsAttention = argv.attention || argv.a;
const shouldShowVersion = argv.version || argv.v;
const shouldShowSimpleOutput = argv.simple || argv.s;
const dirs = argv._.length ? argv._ : [process.cwd()];
const shouldSort = Boolean(argv.sort);
const sortDirection = shouldSort && typeof argv.sort === 'string' ? argv.sort : SORT_DIRECTIONS.ASC;
Boolean(argv.debug) ? printLine : C.NOOP;
let table;
const statuses = [];
UpdateNotifier__default["default"]({
    pkg: { name, version },
    updateCheckInterval: C.updateInterval * C.daysMultiplier,
}).notify();
process.on('uncaughtException', (err) => {
    printLine(`Caught exception: ${err}`);
    process.exit();
});
if (shouldShowVersion) {
    printLine(version);
    process.exit();
}
if (shouldShowHelp) {
    printLine(getHelp());
    process.exit();
}
function getLineForSimpleStatus(status) {
    const str = [];
    for (let i = 0; i < C.simple.length; i++) {
        str.push(status[C.simple[i]]);
    }
    return str.join(',');
}
const prepareSimpleOutput = (statuses) => statuses.map((status) => getLineForSimpleStatus(status));
const finish = (compactness) => () => {
    process.stdout.clearLine(0);
    process.stdout.cursorTo(0);
    if (shouldSort) {
        table.sort(SORT_FUNCTIONS[sortDirection]);
    }
    if (shouldShowSimpleOutput) {
        printLine(prepareSimpleOutput(statuses).join('\n'));
    }
    else {
        printLine(table.toString());
    }
    if (compactness !== C.COMPACTNESS_LEVELS.NONE) {
        const str = [];
        Object.keys(C.headers).map((key) => {
            const header = C.headers[key];
            str.push(chalk__default["default"].cyan(header.short) + ': ' + header.long);
        });
        if (compactness !== C.COMPACTNESS_LEVELS.HIGH) {
            printLine(str.join(', ') + '\n');
        }
    }
};
const listRepos = (_dirs) => {
    const spinner = startSpinner();
    const compactness = getCompactness(argv);
    table = initTable(compactness);
    const [cwd] = _dirs;
    printLine(chalk__default["default"].green(cwd));
    fs__default["default"].readdir(cwd)
        .then((files) => files.map((file) => path__default["default"].resolve(cwd, file)))
        .then((files) => Promise.all(files.map((file) => fs__default["default"].stat(file).then((stat) => ({ path: file, stat }), () => ({ path: file, stat: null })))))
        .then((statuses) => processDirectories(statuses))
        .then((statuses) => {
        statuses.map((status) => pushToTable(table, { showGitOnly, needsAttention })(status));
    })
        .then(() => stopSpinner(spinner))
        .then(() => finish(compactness)())
        .catch((err) => {
        throw err;
    });
};
listRepos(dirs);
